<?php

// $Id: Cache.php,v 1.1.2.4 2010/03/08 21:13:38 fgiasson Exp $

/*! @ingroup ConStructModule */
//@{

/*! 	@file Cache.php
    @brief Instance Record views caching.
    
    @author Frederick Giasson, Structured Dynamics LLC.
    
   \n\n\n
 */

/*! @file Cache.php
   @brief Instance Record views caching.
   @details This class  is intended to be used to cache generated templated views. It yet has to be fully implemented.
  
   \n\n
 
   @author Frederick Giasson, Structured Dynamics LLC.
   
   \n\n\n
 */

/*!	@brief Cache templated instance record views.
    @details This tool is used to access any "views" of a resource of the system. If the cache is still valid, the system will take its rendering from the cache, otherwise it will re-render the instance record.
    
    @todo Implementing the memcached caching tool at this level too (not only in WSF?)
    
    @todo Invalidating the cache for these scenarios: (1) create instance record, (2) update instance record, (3) template creation/update/deletation, (4) deletation of an instance record, (5) change in an ontology description.
    
    @author Frederick Giasson, Structured Dynamics LLC.
*/
class Cache
{
  /*! @brief All resources being re-cached */
  private $resources = array();

  function __construct(){}

  function __destruct(){}


  /*!	 @brief Recreate the cache for all resources described on the node.
              
      \n
      
      @warning This method should only be used with a small number of instance records indexed on the node.
      @note This method is not currently used
    
      @author Frederick Giasson, Structured Dynamics LLC.
    
      \n\n\n
  */
  public function recreateCache()
  {
  /*
  db_query("drop table {conStructtemplates_cache}");
  
  // Recreate the cache table
  db_query("create table {conStructtemplates_cache} (
            id varchar(64),
            uri varchar(2048),
            template_name varchar(1024),
            template_css varchar(1024),
            template_data text,
            invalidate boolean,
            cluster varchar(64),
            primary key (id),
            key `conStructtemplates_cache_uri` (uri)
          )");

  //
  // Get resources from the Browse web service.
  //
  

  // Step #2: generating the templates for all of these resources
  include_once("classHierarchySerialized.php");
  include_once("propertyHierarchySerialized.php");		
  
  foreach($this->resources as $resource)
  {
    $vBinder = new VisualBinder($resource, TRUE, $classHierarchy, $propertyHierarchy);

    $css = $vBinder->getResourceCss();
    
    // Step #3: archiving the template in the cache table
    db_query("insert into {conStructtemplates_cache} (id, uri, template_name, template_css, template_data, invalidate) values ('".md5($resource)."', '$resource', '".$vBinder->templateFile."', '".$css."', '".str_replace("'", "\'", $vBinder->getResourceHtml())."', 0)");
    
    unset($vBinder);
  }
  */
  }

  /*!	 @brief Get a templated resource (from cache or net depending if it is still valid)
              
      \n
      
      @param[in] $uri URI of the instance record to cache or to get the cache.
      @param[in] $dataset URI of the dataset where the resource is indexed
      
      @return returns An array with the HTML code of the templated instance record and its related CSS links.
    
      @author Frederick Giasson, Structured Dynamics LLC.
    
      \n\n\n
  */

  public function getCachedResource($uri, $dataset = "")
  {
    // Step #1: check if the resource is cached
/*
$resultset = db_query("select template_data, template_css, invalidate from {conStructtemplates_cache} where uri = '$uri'");		

while ($nbResources = db_fetch_object($resultset)) 
{
  if($nbResources->invalidate == 0)
  {
    return(array($nbResources->template_data, $nbResources->template_css));		
  }
}
*/
    // Step #2: if it is not, we generate it

    include_once("VisualBinder.php");

    //		include_once("classHierarchySerialized.php");
    //		include_once("propertyHierarchySerialized.php");

    include_once('./' . drupal_get_path('module', 'conStruct') . '/framework/ClassHierarchy.php');
    include_once('./' . drupal_get_path('module', 'conStruct') . '/framework/PropertyHierarchy.php');

    $filename = './' . drupal_get_path('module', 'conStruct') . '/framework/ontologies/classHierarchySerialized.srz';
    $file = fopen($filename, "r");
    $classHierarchy = fread($file, filesize($filename));
    $classHierarchy = unserialize($classHierarchy);
    fclose($file);

    $filename = './' . drupal_get_path('module', 'conStruct') . '/framework/ontologies/propertyHierarchySerialized.srz';
    $file = fopen($filename, "r");
    $propertyHierarchy = fread($file, filesize($filename));
    $propertyHierarchy = unserialize($propertyHierarchy);

    $vBinder = new VisualBinder($uri, $dataset, TRUE, $classHierarchy, $propertyHierarchy);

    // Step #3: cache the result
    $html = $vBinder->getResourceHtml();

    $css = $vBinder->getResourceCss();

//		db_query("insert into {conStructtemplates_cache} (id, uri, template_name, template_css, template_data, invalidate) values ('".md5($uri)."', '$uri', '".$vBinder->templateFile."', '".$css."', '".str_replace("'", "\'", $html)."', 0)");

    unset($vBinder);

    return (array(
      $html,
      $css
    ));
  }

  /*!	 @brief Get the number of cached resources in the node
              
      \n
      
      @return returns The number of cached resources in the node
    
      @author Frederick Giasson, Structured Dynamics LLC.
    
      \n\n\n
  */
  public function getNbCachedResources()
  {
    $resultset = db_query("select count(*) as nb from {conStructtemplates_cache}");

    while ($nbResources = db_fetch_object($resultset))
    {
      return ($nbResources->nb);
    }
  }
}

//@}

?>