<?php

// $Id: TemplatedNode.php,v 1.1.2.3 2010/03/08 19:17:51 jonphipps Exp $

if (is_file(dirname(__FILE__) . "/smarty/Smarty.class.php"))
{
  include_once("smarty/Smarty.class.php");
}
if (is_file(dirname(__FILE__) . "/smarty/libs/Smarty.class.php"))
{
  include_once("smarty/libs/Smarty.class.php");
}

include_once("SmartyAggregate.php");

/*! @ingroup ConStructModule */
//@{

/*! @file TemplatedNode.php
   @brief Inject templated content in a drupal node using PHP
  
   \n\n
 
   @author Frederick Giasson, Structured Dynamics LLC.
   
   \n\n\n
 */

/*!	 @brief Inject templated content in a drupal node using PHP
     @details This class is created and its templated content is injected within a drupal node. The node has to enable PHP scripting within the node to enable this possibility.
            
    \n
    
    @author Frederick Giasson, Structured Dynamics LLC.
  
    \n\n\n
*/
class TemplatedNode
{
  /*! @brief Generated HTML code */
  private $html;

  /*! @brief Smarty templating engine object */
  private $smarty;


  /*!	@brief Constructor
              
      \n
      
      @param[in] $template Name of the template to render.
      @param[in] $dataset Dataset where we have to get the resources to aggregate
      
      @return returns the literal that refers to that label
    
      @author Frederick Giasson, Structured Dynamics LLC.
    
      \n\n\n
  */
  function __construct($template, $dataset)
  {
    $this->smarty = new Smarty();

    $this->smarty->template_dir = realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/";

    $this->smarty->compile_dir =
      realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/templates_c/";

    $this->smarty->config_dir = realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/configs/";

    $aggregate = new SmartyAggregate($dataset);

    $this->smarty->register_object("aggregate", $aggregate);

    $this->html = $this->smarty->fetch($template);

    $pos = stripos($this->html, "<body>") + 6;
    $posEnd = stripos($this->html, "</body>", $pos - 6);
    $this->html = substr($this->html, $pos, $posEnd - $pos);
  }

/*!	@brief Get the erndered HTML content
            
    \n
    
    @return returns the rendered HTML content of the template. The returned content is the content within the &lt;body /&gt; HTML element.
  
    @author Frederick Giasson, Structured Dynamics LLC.
  
    \n\n\n
*/
  public function getHtml()
  {
    // Fix the URI references to reflect the current URI schema used by the user.
    $fixedDomain = str_replace(get_domain(getResourceBaseURI()), get_domain($base_url), getResourceBaseURI());
    $fixedDomain = str_replace("resource", "view", $fixedDomain);

    $this->html = str_replace(getResourceBaseURI(), $fixedDomain, $this->html);

    return ($this->html);
  }
}

//@}

?>