<?php

// $Id: VisualBinder.php,v 1.1.2.10 2010/11/08 21:20:30 fgiasson Exp $

include_once("Resource.php");
include_once("WebServiceQuerier.php");
include_once("ClassHierarchy.php");
include_once("PropertyHierarchy.php");
include_once("Namespaces.php");
if (is_file(dirname(__FILE__) . "/smarty/Smarty.class.php"))
{
  include_once("smarty/Smarty.class.php");
}
if (is_file(dirname(__FILE__) . "/smarty/libs/Smarty.class.php"))
{
  include_once("smarty/libs/Smarty.class.php");
}


/*! @ingroup ConStructModule */
//@{

/*! @file VisualBinder.php
   @brief Bind an instance record to its template

   \n\n

   @author Frederick Giasson, Structured Dynamics LLC.

   \n\n\n

 */

/*!	 @brief Bind an instance record to its template

     @author Frederick Giasson, Structured Dynamics LLC.
*/

class VisualBinder
{
  /*! @brief The URI of the instance record */
  public $uri = "";

  /*! @brief The URI of the dataset where the record is archived */
  public $dataset = "";

  /*! @brief The instance record description */
  public $resource;

  /*! @brief Type of the instance record */
  public $type = "";

  /*! @brief Selected template file */
  public $templateFile = "";

  /*! @brief All available templates on the node */
  public $templates = array();

  /*! @brief Classes hierarchy and descriptions on the node */
  private $classHierarchy;

  /*! @brief Properties hierarchy and descriptions on the node */
  private $propertyHierarchy;

  /*! @brief Smarty processor object */
  private $smarty;

  /*! @brief Tells the visual binder if we add the "links-to" links or not */
  private $linksTo = FALSE;

  /*! @brief Tells the visual binder to add or ignore the labels of the values of the attributes if they are
             resources. In some cases, we don't want these labels to be retreived by the templating engine' */
  private $setAttributeIgnoreResourceLabel = FALSE;

  /*!	 @brief Constructor

      \n

      @param[in] $uri URI of the instance record
      @param[in] $dataset Dataset URI where the resource is indexed
      @param[in] $linksTo Include or exclude "links-to" linkage to this instance record.
      @param[in] $classHierarchy Reference to the class hierarchy structure
      @param[in] $propertyHierarchy Reference to the properties hierarchy structure

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  function __construct($uri, $dataset, $linksTo = FALSE, &$classHierarchy, &$propertyHierarchy)
  {
    $this->uri = $uri;
    $this->dataset = $dataset;

    $this->linksTo = $linksTo;
    $this->classHierarchy = $classHierarchy;
    $this->propertyHierarchy = $propertyHierarchy;

    $this->smarty = new Smarty();

    $this->smarty->template_dir = realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/";
    $this->smarty->compile_dir =
      realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/templates_c/";
    $this->smarty->config_dir = realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/configs/";

    $this->smarty->debugging = true;
    $this->smarty->error_reporting = true;

    // Step 1: getting triples describing the resource.
    $this->resource = new Resource($this->uri, $dataset, TRUE);
    
    // Step 2: getting the list of available templates.
    $this->getTemplates();

    // Step 3: binding the type(s) to the proper template.
    $this->bindTemplate($this->resource->getTypes());

    $fhandle = fopen($this->smarty->template_dir . $this->templateFile, "r");
    $templateContent = fread($fhandle, filesize($this->smarty->template_dir . $this->templateFile));
    fclose($fhandle);

    // Step 4.0: check for the "setting->setAttributeIgnoreResourceLabel" template setting
    preg_match("/.*setting->setAttributeIgnoreResourceLabel=[\s]*true.*/Uim", $templateContent, $matches);
    
    if(count($matches) > 0)
    {
      $this->setAttributeIgnoreResourceLabel = TRUE;
    }

    // Step 4.1: check for "linksto" templates settings
    preg_match("/.*setting->LinksTo=[\s]*false.*/Uim", $templateContent, $matches);


    // Step 5: binding of properties to their visual cluster.
    $this->bindClusters((count($matches) > 0 ? FALSE : TRUE));
  }

  function __destruct(){}

  /*!	@brief Get references to CSS files defined in templates

      \n

      @return returns an array of CSS files URLs

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getResourceCss()
  {
    global $base_url;
    
    $cssLinks = array();

    $templateHtml = $this->smarty->fetch($this->templateFile);
    
    $posHeadStart = stripos($templateHtml, "<head");
    $posHeadEnd = stripos($templateHtml, "</head");
    
    if($posHeadStart !== FALSE)
    {
      $posLinkStart = $posHeadStart;
      
      while(($posLinkStart = stripos($templateHtml, "<link", $posLinkStart)) !== FALSE)
      {
        // Make sure the link is defined in the header of the HTML file
        if($posLinkStart >= $posHeadEnd)
        {
          break;
        }
        
        $posLinkEnd = stripos($templateHtml, ">", $posLinkStart);
        
        $link = substr($templateHtml, $posLinkStart, $posHeadEnd - $posLinkStart);
        
        $posLinkStart = $posLinkEnd;
        
        array_push($cssLinks, $link);
      }
    }
    
    return($cssLinks);
  }

  /*!	 @brief Render the HTML of a given instance record using the appropriate template

      \n

      @return returns the rendered HTML for that given instance record

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getResourceHtml()
  {
    global $base_url;

    $templateHtml = $this->smarty->fetch($this->templateFile);

    // Get templated title.
    $pos = stripos($templateHtml, "<title>") + 7;
    $posEnd = stripos($templateHtml, "</title>", $pos);
    $htmlPageTitle = substr($templateHtml, $pos, $posEnd - $pos);

    $pos = stripos($templateHtml, "<body>") + 6;
    $posEnd = stripos($templateHtml, "</body>", $pos - 6);
    $templateHtml = substr($templateHtml, $pos, $posEnd - $pos);


    // Add administrator pannel
    if (user_access('administer conStruct'))
    {
      $templateHtml = "<span class=\"item-admin-toolbox\" style=\"width: 100%; position: static; margin-top:10px; " .
        "margin-bottom:10px; font-size:small;\">Administration panel: &nbsp;&nbsp;&nbsp;<a href=\"" . $base_url
        . "/conStruct/update/?dataset="
        . urlencode(($this->dataset == "" ? getDatasetFromUri($this->uri) : $this->dataset)) . "&uri="
        . urlencode($this->uri) . "\" alt=\"Update item\" title=\"Update item\"><img src=\"" . $base_url . "/"
        . drupal_get_path("module", "structView") . "/imgs/overlays.png\" border=\"0\" " .
        "style=\"padding-right:5px;\" /></a>&nbsp;&nbsp;&nbsp;<a href=\"" . $base_url .
        "/conStruct/delete/?dataset="
        . urlencode(($this->dataset == "" ? getDatasetFromUri($this->uri) : $this->dataset)) . "&uri="
        . urlencode($this->uri) . "\" alt=\"Delete item\" title=\"Delete item\"><img src=\"" . $base_url . "/"
        . drupal_get_path("module", "structView") . "/imgs/cancel.png\" border=\"0\" " .
        "style=\"\" /></a></span>" . $templateHtml;
    }

    $templateHtml = "<title>$htmlPageTitle</title>" . $templateHtml;
/*
    $templateHtml = str_replace(array(
      "\n",
      "\r"
    ), "", $templateHtml);
*/
    // Removing layout table boxes.
    if (!isset($_GET["showLayoutBoxes"]))
    {
      $templateHtml = preg_replace("/<!\-\-[\s]*layout box start[\s]*\-\->(.*)<!\-\-[\s]*layout box stop[\s]*\-\->/Ui",
        "", $templateHtml);
    }

    // Fix the URI references to reflect the current URI schema used by the user.
    $fixedDomain = str_replace(get_domain(getResourceBaseURI()), get_domain($base_url), getResourceBaseURI());
    $fixedDomain = str_replace("resource", "view", $fixedDomain);

    $templateHtml = str_replace(getResourceBaseURI(), $fixedDomain, $templateHtml);

    return ($templateHtml);
  }

  /*!	 @brief Bind information for an instance record to its Smarty representation

      \n

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  private function bindClusters($getLinksTo = TRUE)
  {
    $properties = $this->resource->getSubjectProperties();

    $attribute = new SmartyAttribute($this->uri, $this->resource->getDataset(), $this->resource->getRawSerialization());

    foreach ($properties as $property)
    {
      $values = $this->resource->getPropertyValues($property);

      // Check if the property is defined in the property hierarchy structure.
      if (isset($this->propertyHierarchy->properties[$property]))
      {
        foreach ($values as $value)
        {
          $attribute->setAttribute($this->propertyHierarchy->properties[$property]->name,
            $this->propertyHierarchy->properties[$property]->label, $value[0], $value[1], $this->resource->getDataset(),
            $this->classHierarchy, (isset($this->resource->reificationTriples[$property][$value[0]]) !== FALSE
            ? $this->resource->reificationTriples[$property][$value[0]] : FALSE), (isset($this->resource->
            reificationTriples[$property][$value[0]]["http://www.w3.org/2000/01/rdf-schema#label"]) !== FALSE
            ? $this->resource->
              reificationTriples[$property][$value[0]]["http://www.w3.org/2000/01/rdf-schema#label"][0] : FALSE), 
              $this->setAttributeIgnoreResourceLabel);
        }
      }
      else
      {
        $pos = strripos($property, "#");

        if ($pos === FALSE)
        {
          $pos = strripos($property, "/");
        }

        if ($pos !== FALSE)
        {
          $pos++;
        }

        $name = substr($property, $pos, strlen($property) - $pos);

        foreach ($values as $value)
        {
          $attribute->setAttribute($property, $name, $value[0], $value[1], $this->resource->getDataset(),
            $this->classHierarchy, (isset($this->resource->reificationTriples[$property][$value[0]]) !== FALSE
            ? $this->resource->reificationTriples[$property][$value[0]] : FALSE), (isset($this->resource->
            reificationTriples[$property][$value[0]]["http://www.w3.org/2000/01/rdf-schema#label"]) !== FALSE
            ? $this->resource->
              reificationTriples[$property][$value[0]]["http://www.w3.org/2000/01/rdf-schema#label"][0] : FALSE),
              $this->setAttributeIgnoreResourceLabel);
        }
      }
    }


    // Links-to Links
    if ($getLinksTo)
    {
      $properties = $this->resource->getObjectProperties();

      foreach ($properties as $property)
      {
        $values = $this->resource->getPropertyValues($property);

        // Check if the property is defined in the property hierarchy structure.
        if (isset($this->propertyHierarchy->properties[$property]))
        {
          foreach ($values as $value)
          {
            $attribute->setAttributeLinksTo($this->propertyHierarchy->properties[$property]->name,
              $this->propertyHierarchy->properties[$property]->label, $value, "", $this->resource->getDataset(),
              $this->classHierarchy);
          }
        }
        else
        {
          $pos = strripos($property, "#");

          if ($pos === FALSE)
          {
            $pos = strripos($property, "/");
          }

          if ($pos !== FALSE)
          {
            $pos++;
          }

          // Save the URI of the class or property passed in parameter
          $name = substr($property, $pos, strlen($property) - $pos);

          foreach ($values as $value)
          {
            $attribute->setAttributeLinksTo($property, $name, $value, "", $this->resource->getDataset(),
              $this->classHierarchy);
          }
        }
      }
    }

    $this->smarty->register_object("attribute", $attribute);
  }

  /*!	 @brief Get all available templates defined on the node.

      \n

      @return returns an array of templates files defined on the node.

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  private function getTemplates()
  {
    if (is_dir(realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/"))
    {
      if ($handle = opendir(realpath(".") . "/" . drupal_get_path("module", "conStruct") . "/templates/"))
      {
        while (($file = readdir($handle)) !== FALSE)
        {
          if ($file != "." && $file != ".." && (stripos($file, ".html") !== FALSE || stripos($file, ".htm") !== FALSE
            || stripos($file, ".xhtml") !== FALSE))
          {
            $this->templates[$file] = 1;
          }
        }
      }
    }
  }

/*!	 @brief Bind an instance record to a template.
     @details If a template is not explecitely defined for this type of instance record, this function will try to find a template for a super-type of the type of the instance record.

    \n


    @author Frederick Giasson, Structured Dynamics LLC.

    \n\n\n
*/
  private function bindTemplate($types)
  {
    if (count($types) < 1)
    {
      return (FALSE);
    }

    // Here is the procedure to bind a resource to a display template
    // 1: check the first type. If it has a templated binded to it, use this one. (it always uses the first one found)
    // 2: iterates for all types until you get one that has a related template.
    // 3: if none have a binded template, then we check their super-classes
    // 4: if none have a binded template, then we bind to the "generic" template.

    // Step 1 & 2
    $found = FALSE;
    foreach ($types as $type)
    {
      foreach ($type as $t)
      {
        $this->getTypeTemplate($t);

        if ($this->templateFile != "")
        {
          $this->type = $t;
          
          if($this->templateFile != "owl_thing.html")
          {
            $found = TRUE;
          }
          
          break;
        }
      }
      
      if($found == TRUE)
      {
        break;
      }
    }

    // Step 3
    if ($this->templateFile == "")
    {
      foreach ($types as $type)
      {
        foreach ($type as $t)
        {
          if (isset($this->classHierarchy))
          {
            $superClasses = $this->classHierarchy->getSuperClasses($t);

            foreach ($superClasses as $sc)
            {
              $this->getTypeTemplate($sc->name);

              if ($this->templateFile != "")
              {
                $this->type = $t;
                break;
              }
            }

            if ($this->templateFile != "")
            {
              break;
            }
          }
        }
      }
    }

    // Step 4
    if ($this->templateFile == "")
    {
      $this->type = $types[0][0];
      $this->templateFile = "owl_thing.html";
    }
  }

  private function getTypeTemplate($type)
  {
    $templateName = strtolower(str_replace(":", "_", get_uri_label($type))) . ".html";

    if (isset($this->classHierarchy->classes[$type]) && $templateName != "" && isset($this->templates[$templateName]))
    {
      $this->templateFile = $templateName;
    }
  }
}

/*!	 @brief A Smarty resource
     @details This class is used by Smarty to get information abuot a given instance record. This is what we call the "Smarty API". Each of these methods are then assigned to Smarty variables within the templates definitions.

    \n

    @author Frederick Giasson, Structured Dynamics LLC.

    \n\n\n
*/
class SmartyAttribute
{
  /*! @brief Array of attributes-value pairs used by the Smarty template at rendering time */
  private $attributes;

  /*! @brief Array of unassigned attributes that describe an instance record but that are not defined in the template */
  private $unassignedAttributes = array();

  /*! @brief Array of attributes that links to this instance record */
  private $attributesLinksTo = array();

  /*! @brief Identifier (URI) of the instance record */
  private $uri;

  /*! @brief URI of the dataset where this instance record is archived */
  private $dataset;

  /*! @brief Raw serialization data describing the record (text/xml) */
  private $rawSerialization;

  /*!	 @brief Construct

      \n

      @param[in] $uri Identifier of the instance record
      @param[in] $dataset Dataset URI where the resource is indexed

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  function __construct($uri, $dataset = "", $rawSerialization)
  {
    $this->uri = $uri;
    $this->dataset = $dataset;
    $this->rawSerialization = $rawSerialization;
  }

  /*!	 @brief Get the identifier of the instance record

      \n

      @param[in] $params Unused
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns the identifer of the instance record

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getURI($params, &$smarty_obj)
  {
    return ($this->uri);
  }

/*!	 @brief Get a specific attribute of another resource. Example, the URL of a homepage resource.

    \n

    @param[in] $params Smarty parameters that come from the template definition. "property": the target property to get the value from; "resource": the URI of the target resource.
    @param[in] $smarty_obj reference to the smarty object being processed

    @return returns the value of that resource.

    @author Frederick Giasson, Structured Dynamics LLC.

    \n\n\n
*/
  public function getResourcePropertyValues($params, &$smarty_obj)
  {
    $resource = new Resource($params["resource"], $params["dataset"], FALSE);

    $value = $resource->getPropertyValues(get_label_uri($params["property"]));

    return (array(array(
      "value" => $value[0][0],
      "valueType" => $value[0][1],
      "valueLiteral" => $value[0][0],
      "attribute" => $params["property"],
      "attributeLabel" => ""
    )));
  }


  /*!	 @brief Get a "label" that refers to an instance record

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns the literal that refers to that label

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getLabel($params, &$smarty_obj)
  {
    if (isset($this->attributes[$params["attribute"]]))
    {
      $this->unassignedAttributes[$params["attribute"]] = 0;
      return ($this->attributes[$params["attribute"]][0]["attributeLabel"]);
    }
    else
    {
      return ("");
    }
  }

  /*!	 @brief Get all values for a given attribute

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns an array of values for a given attribute

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getValues($params, &$smarty_obj)
  {
    $values = array();

    /* if the attribute exists, we make sure it is not get assigned to the "unassibned" attributes list */
    if (isset($this->attributes[$params["attribute"]]))
    {
      $this->unassignedAttributes[$params["attribute"]] = 0;
    }

    /* populate the array of values for this attribute to return to the template */
    foreach ($this->attributes[$params["attribute"]] as $value)
    {
      array_push($values, array(
        "value" => $value["value"],
        "valueType" => $value["valueType"],
        "valueLiteral" => $value["valueLiteral"],
        "reify" => $value["reify"]
      ));
    }

    return ($values);
  }

  /*!	 @brief Get unasigned attributes describing a given instance records that are not defined in a template

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns a list of attributes-values pair of unsigned attributes

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getUnasigned($params, &$smarty_obj)
  {
    $values = array();

    foreach ($this->unassignedAttributes as $attribute => $assignation)
    {
      if ($assignation == 1)
      {
        foreach ($this->attributes[$attribute] as $value)
        {
          array_push($values, array(
            "value" => $value["value"],
            "valueType" => $value["valueType"],
            "valueLiteral" => $value["valueLiteral"],
            "attribute" => $value["attribute"],
            "attributeLabel" => $value["attributeLabel"],
            "reify" => $value["reify"]
          ));
        }
      }
    }

    return ($values);
  }

  /*!	 @brief Get links-to attributes that links to a given instance record

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns a list of attributes-values pair linking to this instance record

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getLinksTo($params, &$smarty_obj)
  {
    $values = array();

    foreach ($this->attributesLinksTo as $attribute)
    {
      foreach ($attribute as $value)
      {
        array_push($values, array(
          "referrer" => $value["value"],
          "referrerLabel" => $value["valueLiteral"],
          "attribute" => $value["attribute"],
          "attributeLabel" => $value["attributeLabel"]
        ));
      }
    }

    return ($values);
  }

  /*!	 @brief Get a "label" that refers to an instance record

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns the literal that refers to that label

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getInstanceRecordLabel($params, &$smarty_obj)
  {
    $labelProperties = array(
      Namespaces::$iron . "prefLabel",
      Namespaces::$iron . "altLabel",
      Namespaces::$asn . "statementLabel",
      Namespaces::$dc . "title",
      Namespaces::$dcterms . "title",
      Namespaces::$foaf . "name",
      Namespaces::$foaf . "givenname",
      Namespaces::$foaf . "family_name",
      Namespaces::$rdfs . "label",
      Namespaces::$skos_2004 . "prefLabel",
      Namespaces::$skos_2004 . "altLabel",
      Namespaces::$skos_2008 . "prefLabel",
      Namespaces::$skos_2008 . "altLabel",
      Namespaces::$doap . "name",
      Namespaces::$geoname."name"
      
    );

    foreach ($labelProperties as $property)
    {
      if (isset($this->attributes[$property]))
      {
        $this->unassignedAttributes[$property] = 0;

        return ($this->attributes[$property][0]["valueLiteral"]);
      }
    }

    return ("");
  }

  /*!	 @brief Get a "type" that refers to an instance record

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns the literal that refers to this type

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getInstanceRecordType($params, &$smarty_obj)
  {
    if (isset($this->attributes["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"]))
    {
      $this->unassignedAttributes["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"] = 0;

      return ($this->attributes["http://www.w3.org/1999/02/22-rdf-syntax-ns#type"][0]["valueLiteral"]);
    }
    else
    {

      return ("");
    }
  }

  /*!	 @brief Get a "type" that refers to an instance record

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns the literal that refers to this type

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getInstanceRecordDataset($params, &$smarty_obj)
  {
    return ($this->dataset);
  }

  /*!	 @brief Get a "description" that refers to an instance record

      \n

      @param[in] $params Smarty parameters that come from the template definition
      @param[in] $smarty_obj reference to the smarty object being processed

      @return returns the literal that refers to this description

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function getInstanceRecordDescription($params, &$smarty_obj)
  {
    $descriptionProperties = array(
      Namespaces::$iron . "description",
      Namespaces::$dcterms . "description",
      Namespaces::$skos_2004 . "definition",
      Namespaces::$skos_2008 . "definition",
      Namespaces::$bio . "olb",
      Namespaces::$bibo . "abstract",
      Namespaces::$doap . "description",
      Namespaces::$geoname."name"
    );

    foreach ($descriptionProperties as $property)
    {
      if (isset($this->attributes[$property]))
      {
        $this->unassignedAttributes[$property] = 0;

        return ($this->attributes[$property][0]["valueLiteral"]);
      }
    }

    return ("");
  }

  /*!	 @brief Set an attribute-value pair for this instance-record

      \n

      @param[in] $attribute URI of the attribute
      @param[in] $attributeLabel Label describing the attribute
      @param[in] $value Value of the attribute
      @param[in] $valueType Type of the value of the attribute (can be: (1) literal (2) resource)
      @param[in] $valueDataset Dataset where the value can be indexed (if the value is another resource)
      @param[in] $classHierarchy Class hierarchy of the system
      @param[in] $reifiedStatements Array of reified statements for that attribute/value pair
      @param[in] $reifiedObjectLabel Array of reified objects that are literals for that attribute/value pair

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function setAttribute($attribute, $attributeLabel, $value, $valueType, $valueDataset, &$classHierarchy,
    $reifiedStatements, $reifiedObjectLabel, $setAttributeIgnoreResourceLabel)
  {
    $attributeKey = $attribute;
    $valueLabel = $value;

    if ($valueType == "")
    {
      $valueType = "resource";

      // If it is a resource, we try to get its label.

      // First check if it is a class
      if (isset($classHierarchy->classes[$value]))
      {
        $valueLabel = $classHierarchy->classes[$value]->label;
      }
      elseif ($attribute == "http://www.w3.org/1999/02/22-rdf-syntax-ns#type")
      {
// If the attribute is rdf:Type, then we try to find the label of the type within the classHierarchy, if not available
// we use the end of the URI of the type as the label.

        /*! @todo Checking in the classHierarchy of the system for the label of this type */

        if (FALSE /* check if it is in the classHierarchy */){}
        else
        {
          $pos = strrpos($value, "/");

          $valueLabel = substr($value, $pos + 1, strlen($value) - $pos);
        }
      }
      elseif ($reifiedObjectLabel !== FALSE)
      {
// Check if there is a UI "label" reification statement for this resource. If there is one, then we use it
// instead of anything else. This mean that even if the label of the resource is "A", and if there is a "B" reification
// statement, the user want its data being displayed with "B" label.

        $valueLabel = $reifiedObjectLabel;
      }
      else
      {
        // Get the label from the triple store.
        if($setAttributeIgnoreResourceLabel === FALSE)
        {
          $resource = new Resource($value, $valueDataset, FALSE);

          $valueLabel = $resource->getLabel();
          $valueLabel = $valueLabel[0];
        }
      }
    }
    else
    {
      $valueType = "literal";
    }

    if (!isset($this->attributes[$attributeKey]))
    {
      $this->attributes[$attributeKey] = array(array(
        "attribute" => $attribute,
        "attributeLabel" => $attributeLabel,
        "value" => $value,
        "valueType" => $valueType,
        "valueLiteral" => $valueLabel,
        "reify" => $reifiedStatements
      ));
    }
    else
    {
      array_push($this->attributes[$attributeKey], array(
        "attribute" => $attribute,
        "attributeLabel" => $attributeLabel,
        "value" => $value,
        "valueType" => $valueType,
        "valueLiteral" => $valueLabel,
        "reify" => $reifiedStatements
      ));
    }

    $this->unassignedAttributes[$attribute] = 1;
  }

  /*!	 @brief Set an attribute-value pair that links to this instance-record

      \n

      @param[in] $attribute URI of the attribute
      @param[in] $attributeLabel Label describing the attribute
      @param[in] $value Value of the attribute
      @param[in] $valueType Type of the value of the attribute (can be: (1) literal (2) resource)
      @param[in] $valueDataset Dataset where the value can be indexed (if the value is another resource)
      @param[in] $classHierarchy Class hierarchy of the system

      @author Frederick Giasson, Structured Dynamics LLC.

      \n\n\n
  */
  public function setAttributeLinksTo($attribute, $attributeLabel, $value, $valueType, $valueDataset, &$classHierarchy)
  {
    $attributeKey = $attribute;
    $valueLabel = $value;

    if ($valueType == "")
    {
      $valueType = "resource";

      // If it is a resource, we try to get its label.

      // First check if it is a class
      if (isset($classHierarchy->classes[$value]))
      {
        $valueLabel = $classHierarchy->classes[$value]->label;
      }
      else
      {
        // Get the label from the triple store.
        $resource = new Resource($value, $valueDataset, FALSE);

        $valueLabel = $resource->getLabel();
        $valueLabel = $valueLabel[0];
      }
    }
    else
    {
      $valueType = "literal";
    }

    if (!isset($this->attributesLinksTo[$attributeKey]))
    {
      $this->attributesLinksTo[$attributeKey] = array(array(
        "attribute" => $attribute,
        "attributeLabel" => $attributeLabel,
        "value" => $value,
        "valueType" => $valueType,
        "valueLiteral" => $valueLabel
      ));
    }
    else
    {
      array_push($this->attributesLinksTo[$attributeKey], array(
        "attribute" => $attribute,
        "attributeLabel" => $attributeLabel,
        "value" => $value,
        "valueType" => $valueType,
        "valueLiteral" => $valueLabel
      ));
    }
  }

  public function getAttributes()
  {
    return ($this->attributes);
  }   
  
  /*!   @brief Get Raw data (text/xml) describing the resource. This is normally used by the template to
               send the raw description of the resource to an external module (ex: flex application).
              
      \n
      
      @return returns Returns a string with the raw text/xml description data of the resource.
    
      @author Frederick Giasson, Structured Dynamics LLC.
    
      \n\n\n
  */    
  public function getResourceRawSerialization()
  {
    return ($this->rawSerialization);
  }  
}

//@}

?>